/*******************************************************************************
* File Name: PGA_follower.c
* Version 1.80
*
* Description:
*  This file provides the source code to the API for OpAmp (Analog Buffer) 
*  Component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "PGA_follower.h"
#include <CyLib.h>

uint8 PGA_follower_initVar = 0u;


/*******************************************************************************   
* Function Name: PGA_follower_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the 
*  customizer of the component placed onto schematic. Usually called in 
*  PGA_follower_Start().
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void PGA_follower_Init(void) 
{
    PGA_follower_SetPower(PGA_follower_DEFAULT_POWER);
}


/*******************************************************************************   
* Function Name: PGA_follower_Enable
********************************************************************************
*
* Summary:
*  Enables the OpAmp block operation
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void PGA_follower_Enable(void) 
{
    /* Enable negative charge pumps in ANIF */
    PGA_follower_PUMP_CR1_REG  |= (PGA_follower_PUMP_CR1_CLKSEL | PGA_follower_PUMP_CR1_FORCE);

    /* Enable power to buffer in active mode */
    PGA_follower_PM_ACT_CFG_REG |= PGA_follower_ACT_PWR_EN;

    /* Enable power to buffer in alternative active mode */
    PGA_follower_PM_STBY_CFG_REG |= PGA_follower_STBY_PWR_EN;
}


/*******************************************************************************
* Function Name:   PGA_follower_Start
********************************************************************************
*
* Summary:
*  The start function initializes the Analog Buffer with the default values and 
*  sets the power to the given level. A power level of 0, is same as 
*  executing the stop function.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  PGA_follower_initVar: Used to check the initial configuration, modified 
*  when this function is called for the first time.
*
*******************************************************************************/
void PGA_follower_Start(void) 
{
    if(PGA_follower_initVar == 0u)
    {
        PGA_follower_initVar = 1u;
        PGA_follower_Init();
    }

    PGA_follower_Enable();
}


/*******************************************************************************
* Function Name: PGA_follower_Stop
********************************************************************************
*
* Summary:
*  Powers down amplifier to lowest power state.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void PGA_follower_Stop(void) 
{
    /* Disable negative charge pumps for ANIF only if one ABuf is turned ON */
    if(PGA_follower_PM_ACT_CFG_REG == PGA_follower_ACT_PWR_EN)
    {
        PGA_follower_PUMP_CR1_REG &= ~(PGA_follower_PUMP_CR1_CLKSEL | PGA_follower_PUMP_CR1_FORCE);
    }
    
    /* Disable power to buffer in active mode template */
    PGA_follower_PM_ACT_CFG_REG &= ~PGA_follower_ACT_PWR_EN;

    /* Disable power to buffer in alternative active mode template */
    PGA_follower_PM_STBY_CFG_REG &= ~PGA_follower_STBY_PWR_EN;
}


/*******************************************************************************
* Function Name: PGA_follower_SetPower
********************************************************************************
*
* Summary:
*  Sets power level of Analog buffer.
*
* Parameters: 
*  power: PSoC3: Sets power level between low (1) and high power (3).
*         PSoC5: Sets power level High (0)
*
* Return:
*  void
*
**********************************************************************************/
void PGA_follower_SetPower(uint8 power) 
{
    /* Only High power can be used in PSoC5 */
    #if CY_PSOC5A
        CYASSERT(power == PGA_follower_HIGHPOWER);
    #endif   /* (CY_PSOC5A) */

    PGA_follower_CR_REG = ((PGA_follower_CR_REG & ~PGA_follower_PWR_MASK) | 
                               ( power & PGA_follower_PWR_MASK));   /* Set device power */
}


/* [] END OF FILE */
